#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os
import locale

import gtk, gobject
from gtk import TRUE, FALSE

from bookmarks import *
from utils import *
from bookbuffer import *
from config import charsets_list
import filters

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class BookmarksWindow:

    COLUMN_FOLDER     = 0
    COLUMN_FILENAME   = 1
    COLUMN_AUTHOR     = 2
    COLUMN_NAME       = 3
    COLUMN_PERCENT    = 4
    COLUMN_DECRIPTION = 5
    COLUMN_PATH       = 6 # invisible: path to file
    COLUMN_MARK       = 7 # invisible: mark number

    hide_bookmarks = 0
    notebook_current_page = 0

    def __init__(self, bookmarks):


        glade_file=get_glade_file()
        self.wTree=gtk.glade.XML(glade_file, 'bookmarks_window')

        self.categories_list = bookmarks.categories_list
        self.bookmarks = bookmarks

        self.create_window()

##----------------------------------------------------------------------

    def create_window(self):

        self.window = self.wTree.get_widget('bookmarks_window')

        dic = {
            'hide_bookmarks_toggled': self.hide_bookmarks_toggled_cb,
            'apply_cliced_cb'       : self.apply_cliced_cb,
            'close_clicked_cb'      : lambda x: self.window.destroy(),
            'delete_clicked_cb'     : self.delete_clicked_cb,
            'add_category_cb'       : self.add_category_cb,
            'load_cliced_cb'        : lambda x: self.emit_load_file(),
            'add_file_cb'           : self.add_file_cb,
            'notebook_switch_page_cb': self.notebook_switch_page_cb,
            }

        self.wTree.signal_autoconnect(dic)

        self.paned = self.wTree.get_widget('paned')
        self.category_menu = self.wTree.get_widget('category_menu')
        self.encoding_menu = self.wTree.get_widget('encoding_menu')
        self.file_label = self.wTree.get_widget('file_label')
        self.author_entry = self.wTree.get_widget('author_entry')
        self.name_entry = self.wTree.get_widget('name_entry')

        description_view = self.wTree.get_widget('description_view')
        self.description_buffer = BookBuffer()
        description_view.set_buffer(self.description_buffer)

        preview_view = self.wTree.get_widget('preview_view')
        self.text_buffer = BookBuffer()
        preview_view.set_buffer(self.text_buffer)


        # -- treeview --
        treeview = self.wTree.get_widget('books_treeview')

        #treeview.set_search_column(self.COLUMN_FILENAME) # ???
        treeview.set_rules_hint(TRUE) #   

        # category
        column = gtk.TreeViewColumn('Category', gtk.CellRendererText(),
                                    text=self.COLUMN_FOLDER)
        #column.set_sort_column_id(self.COLUMN_FOLDER)
        column.set_resizable(TRUE)
        column.set_sort_column_id(self.COLUMN_FOLDER)
        treeview.append_column(column)
        # filename
        column = gtk.TreeViewColumn('File', gtk.CellRendererText(),
                                    text=self.COLUMN_FILENAME)
        #column.set_sort_column_id(self.COLUMN_FILENAME)
        column.set_resizable(TRUE)
        column.set_sort_column_id(self.COLUMN_FILENAME)
        treeview.append_column(column)
        # author
        column = gtk.TreeViewColumn('Author', gtk.CellRendererText(),
                                    text=self.COLUMN_AUTHOR)
        column.set_resizable(TRUE)
        column.set_sort_column_id(self.COLUMN_AUTHOR)
        treeview.append_column(column)
        # name
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(),
                                    text=self.COLUMN_NAME)
        column.set_resizable(TRUE)
        column.set_sort_column_id(self.COLUMN_NAME)
        treeview.append_column(column)
        # percent
        column = gtk.TreeViewColumn('%', gtk.CellRendererText(),
                                    text=self.COLUMN_PERCENT)
        column.set_sort_column_id(self.COLUMN_PERCENT)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # description
        column = gtk.TreeViewColumn('Description', gtk.CellRendererText(),
                                    text=self.COLUMN_DECRIPTION)
        column.set_resizable(TRUE)
        treeview.append_column(column)

        treeview.connect('row-activated', lambda w, row, col:
                         self.emit_load_file())

        # -- treview selection --
        selection = treeview.get_selection()
        selection.connect('changed', lambda w: self.treeview_show_selected())
        selection.set_mode(gtk.SELECTION_MULTIPLE)

        self.treeview = treeview

        # -- model --
        model = gtk.TreeStore(gobject.TYPE_STRING, # category
                              gobject.TYPE_STRING, # filename
                              gobject.TYPE_STRING, # author
                              gobject.TYPE_STRING, # name
                              gobject.TYPE_INT,    # percent
                              gobject.TYPE_STRING, # description
                              gobject.TYPE_STRING, # path
                              gobject.TYPE_STRING) # mark number
        self.model = model

        # -- filters --
        filters_table = self.wTree.get_widget('filters_table')

        i=0
        self.filters_check_list = []
        for fltr in filters.filters:

            check = gtk.CheckButton(filters.filters[fltr])
            #check.set_active(parent.text_widget.scroll_type[i][0])
            cb_id = check.connect('clicked', self.filters_toggled, fltr)
            #check.connect('button_release_event', self.filters_toggled, fltr)
            filters_table.attach(check, 0, 4, i, i+1,
                                 xoptions=gtk.FILL, yoptions=gtk.FILL,
                                 xpadding=4, ypadding=4)
            self.filters_check_list.append((fltr, check, cb_id))
            i+=1

        label = gtk.Label('External filter: ')
        filters_table.attach(label, 0, 1, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)

        self.ext_filter_label = gtk.Label('None')
        filters_table.attach(self.ext_filter_label, 1, 2, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)

        button = gtk.Button('Change')
        filters_table.attach(button, 2, 3, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)
        button.connect('clicked', self.ext_filter_clicked_cb)

        filters_table.show_all()

        # -- charsets --
        self.encoding_submenu = gtk.Menu()
        for i in charsets_list:
            item = gtk.MenuItem(i)
            self.encoding_submenu.append(item)
            item.connect_object('activate', self.encodind_submenu_activate_cb, i)
            item.show()
        self.encoding_menu.set_menu(self.encoding_submenu)

        # -- categories --
        self.update_category_menu()

        if self.hide_bookmarks:
            self.wTree.get_widget('hide_bookmarks_button').set_active(TRUE)
        else:
            self.update_treeview()

##----------------------------------------------------------------------

    def get_selected_list(self):

        sel = self.treeview.get_selection()

        selected_list = []
        sel.selected_foreach(lambda model, path, iter:
                             selected_list.append((model, path, iter)))

        return selected_list

##----------------------------------------------------------------------

    def hide_bookmarks_toggled_cb(self, w):
        self.hide_bookmarks = w.get_active()
        self.update_treeview()

##----------------------------------------------------------------------

    def notebook_switch_page_cb(self, w, p, page):
        self.notebook_current_page = page
        self.treeview_show_selected()

##----------------------------------------------------------------------

    def encodind_submenu_activate_cb(self, enc):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]
        filename = model.get_value(iter, self.COLUMN_PATH)
        b = self.bookmarks.get_book(filename)

        if b:
            b.encoding = enc
            self.treeview_show_selected()

##----------------------------------------------------------------------

    def category_submenu_activate_cb(self, new_cat):

        selected_list = self.get_selected_list()

        #if len(selected_list) != 1: return
        for model, path, iter in selected_list:
            #model, path, iter = selected_list[0]
            filename = model.get_value(iter, self.COLUMN_PATH)
            if not filename: return
            self.bookmarks.move_book(filename, new_cat)

        self.update_treeview()

##----------------------------------------------------------------------

    def treeview_show_selected(self):
        selected_list = self.get_selected_list()
        if len(selected_list) != 1:
            self.clear_view()
            return

        model, path, iter = selected_list[0]

        filename = model.get_value(iter, self.COLUMN_PATH)

        if filename:
            try:
                fd =  open(filename)
            except IOError, err:
                self.clear_view()
                error_dialog(self.window, 'Can\'t open file %s: %s' \
                             % (filename, err[1]))
                return
            else:
                fd.close()

        mark = model.get_value(iter, self.COLUMN_MARK)

        found = 0
        b = self.bookmarks.get_book(filename)
        if b:

            self.file_label.set_text(fix_filename(b.filename))

            if mark:
                m = self.bookmarks.get_mark(b, int(mark))
                if m:


                    if self.notebook_current_page == 0:
                        if m.author:
                            self.author_entry.set_text(m.author)
                        else:
                            self.author_entry.set_text(b.author)
                        if m.name:
                            self.name_entry.set_text(m.name)
                        else:
                            self.name_entry.set_text(b.name)
                        self.description_buffer.set_text(m.description)

                    elif self.notebook_current_page == 1:
                        # update text preview
                        self.text_buffer.set_book(b)
                        self.text_buffer.preview(m.offset)
            else:

                if self.notebook_current_page == 0:

                    self.author_entry.set_text(b.author)
                    self.name_entry.set_text(b.name)
                    self.description_buffer.set_text(b.description)

                elif self.notebook_current_page == 1:
                    # update text preview
                    self.text_buffer.set_book(b)
                    self.text_buffer.preview()


            if self.notebook_current_page == 0:
                # update encoding menu
                n = 0
                for enc in charsets_list:
                    if b.encoding == enc:
                        break
                    n = n+1
                # bug ?
                self.encoding_menu.remove_menu()
                self.encoding_submenu.set_active(n)
                self.encoding_menu.set_menu(self.encoding_submenu)

                # update category menu
                n = 0
                for cat in self.categories_list:
                    if b in self.categories_list[cat]:
                        break
                    n = n+1
                self.category_menu.remove_menu()
                self.category_submenu.set_active(n)
                self.category_menu.set_menu(self.category_submenu)

            elif self.notebook_current_page == 2:
                # update filters
                for c in self.filters_check_list:
                    c[1].handler_block(c[2])
                    c[1].set_active(FALSE)
                    if b.internalfilters:
                        for f in b.internalfilters:
                            if f == c[0]:
                                c[1].set_active(TRUE)
                                break
                    c[1].handler_unblock(c[2])
                self.ext_filter_label.set_text(str(b.externalfilter))

            found = 1

        if not found: self.clear_view()

##----------------------------------------------------------------------

    def add_file_cb(self, w):

        dialog = OpenFileDialog(self.window, 1) # from utils module

        filename = dialog.filename
        files_list = dialog.files_list
        dir = os.path.dirname(filename) + os.sep

        if files_list:

            for f in files_list:
                path = os.path.join(dir, f)
                enc = locale.getlocale()[1]
                path = unicode(path, 'utf-8').encode(enc)
                try:
                    fd = open(path)
                except IOError, err:
                    error_dialog(self.window, 'Can\'t open file ',
                                 '%s: %s' % (path, err[1]))
                    continue
                fd.close()

                if not self.bookmarks.get_book(path):
                    book = self.text_buffer.new_book(path)
                    self.bookmarks.add_new_book(book)

            self.update_treeview()
            self.window.emit('bookslist-changed')

##----------------------------------------------------------------------

    def add_category_cb(self, w, a=None):

        model = self.treeview.get_model()

        cat_name = entry_dialog(self.window, 'New category', 'Category: ')

        if cat_name:

            if self.bookmarks.categories_list.has_key(cat_name):
                error_dialog(self, 'Name must be unique')

            else:

                cat_iter = self.model.append(None)
                self.model.set(cat_iter, self.COLUMN_FOLDER, cat_name)
                self.bookmarks.make_new_category(cat_name)

                self.update_category_menu()

##----------------------------------------------------------------------

    def apply_cliced_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

##         model, path, iter = selected_list[0]
##         filename = model.get_value(iter, self.COLUMN_PATH)

        self.change_apply()
        #self.emit('book-changed', filename) # ???

##----------------------------------------------------------------------

    def filters_toggled(self, w, fltr):
        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]
        filename = model.get_value(iter, self.COLUMN_PATH)
        b = self.bookmarks.get_book(filename)
        active = w.get_active()

        if b:
            if not b.internalfilters:
                b.internalfilters = [fltr]
            elif active and not fltr in b.internalfilters:
                if b.internalfilters:
                    b.internalfilters.append(fltr)
                else:
                    b.internalfilters = [fltr]
            elif fltr in b.internalfilters:
                b.internalfilters.remove(fltr)


        #self.change_apply()
        #self.emit('book-changed', filename) # ???

##----------------------------------------------------------------------

    def ext_filter_clicked_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        command = entry_dialog(self.window,
                               'External command', 'External command: ')

        if command:

            model, path, iter = selected_list[0]
            filename = model.get_value(iter, self.COLUMN_PATH)

            b = self.bookmarks.get_book(filename)
            if b:
                b.externalfilter = command
                self.treeview_show_selected()
                #self.update_preview()

##----------------------------------------------------------------------

    def change_apply(self):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        b = self.bookmarks.get_book(filename)

        if b:

            if mark: # selected mark rows
                m = self.bookmarks.get_mark(b, int(mark))
                if m:

                    m.author = self.author_entry.get_text()
                    m.name = self.name_entry.get_text()
                    start, end = self.description_buffer.get_bounds()
                    m.description = self.description_buffer.get_text(start,
                                                                     end, TRUE)
                    self.update_treeview()
            else:

                b.author = self.author_entry.get_text()
                b.name = self.name_entry.get_text()
                start, end = self.description_buffer.get_bounds()
                b.description = self.description_buffer.get_text(start,
                                                                 end, TRUE)
                self.update_treeview()

        self.clear_view()

##----------------------------------------------------------------------

    def emit_load_file(self):

        selected_list = self.get_selected_list()

        if len(selected_list) == 0:
            return

        model, path, iter = selected_list[0]

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        if mark:
            b = self.bookmarks.get_book(filename)
            if b:
                m = self.bookmarks.get_mark(b, int(mark))
                if m:
                    b.current_position = m.position
                    b.current_offset = m.offset
                    self.window.emit('load-file', filename)

        elif filename:
            self.window.emit('load-file', filename)

##----------------------------------------------------------------------

    def delete_clicked_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) == 0:
            return

        # remove mark
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if mark: # mark
                self.bookmarks.remove_bookmark(filename, int(mark))

        # remove book
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if not mark and filename: # book

                if self.bookmarks.remove_book(filename):
                    self.window.emit('book-removed', filename)

        # remove category
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if not mark and not filename: # category

                cat = model.get_value(iter, self.COLUMN_FOLDER)

                if len(self.bookmarks.categories_list[cat]) != 0:
                    error_dialog(self.window, 'Category must be empty')
                else:
                    del self.bookmarks.categories_list[cat]
                    self.update_category_menu()

        self.clear_view()
        self.update_treeview()
        self.window.emit('bookmarks-changed', filename)

##----------------------------------------------------------------------

    def clear_view(self):

        self.file_label.set_text('')
        self.author_entry.set_text('')
        self.name_entry.set_text('')
        self.description_buffer.set_text('')
        self.text_buffer.set_text('')

##----------------------------------------------------------------------

    def update_category_menu(self):
        # update categories menu
        self.category_submenu = gtk.Menu()
        if len(self.categories_list) == 0: # add 'Default' item
            i = 'Default'
            item = gtk.MenuItem(i)
            self.category_submenu.append(item)
            item.connect_object('activate',
                                self.category_submenu_activate_cb, i)
            item.show()
        else:
            for i in self.categories_list:
                item = gtk.MenuItem(i)
                self.category_submenu.append(item)
                item.connect_object('activate',
                                    self.category_submenu_activate_cb, i)
                item.show()
        self.category_menu.set_menu(self.category_submenu)

##----------------------------------------------------------------------

    def update_treeview(self):

        model = self.model
        model.clear()

        if not self.hide_bookmarks:

            for cat in self.categories_list: # categories
                iter = model.append(None)
                model.set(iter, self.COLUMN_FOLDER, cat)

                for book in self.categories_list[cat]: # books
                    filename = fix_filename(os.path.basename(book.filename))
                    book_iter = model.append(iter)

                    desc = fix_description(book.description, 100)
                    model.set(book_iter,
                              #self.COLUMN_FILENAME, os.path.basename(book.filename),
                              self.COLUMN_FILENAME, filename,
                              self.COLUMN_NAME, book.name,
                              self.COLUMN_AUTHOR, book.author,
                              self.COLUMN_PERCENT, int(book.current_position*100),
                              self.COLUMN_DECRIPTION, desc,
                              self.COLUMN_PATH, book.filename)

                    for mark in book.marks_list: # marks
                        mark_iter = model.append(book_iter)
                        desc = fix_description(mark.description, 100)
                        model.set(mark_iter,
                                  self.COLUMN_NAME, mark.name,
                                  self.COLUMN_AUTHOR, mark.author,
                                  self.COLUMN_PERCENT, int(mark.position*100),
                                  self.COLUMN_DECRIPTION, desc,
                                  self.COLUMN_PATH, book.filename,
                                  self.COLUMN_MARK, mark.number)

        else: # self.hide_bookmarks == 1
            for cat in self.categories_list: # categories
                for book in self.categories_list[cat]: # books
                    filename = fix_filename(os.path.basename(book.filename))
                    book_iter = model.append(None)

                    desc = fix_description(book.description, 100)
                    model.set(book_iter,
                              self.COLUMN_FOLDER, cat,
                              self.COLUMN_FILENAME, filename,
                              self.COLUMN_NAME, book.name,
                              self.COLUMN_AUTHOR, book.author,
                              self.COLUMN_PERCENT, int(book.current_position*100),
                              self.COLUMN_DECRIPTION, desc,
                              self.COLUMN_PATH, book.filename)


        self.treeview.set_model(model)

        self.treeview.expand_all()

##----------------------------------------------------------------------
##----------------------------------------------------------------------


gobject.signal_new('load-file', gtk.Widget,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('book-removed', gtk.Widget,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('book-changed', gtk.Widget,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('bookmarks-changed', gtk.Widget,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('bookslist-changed', gtk.Widget,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   ())

##----------------------------------------------------------------------

if(__name__=='__main__'):

    bookmarks = Bookmarks()
    bw = BookmarksWindow(bookmarks)
    bw.set_default_size(600, 600)
    bw.paned.set_position(300)
    bw.connect('destroy', lambda w: gtk.main_quit())
    bw.show_all()
    gtk.main()

