#!/usr/bin/env python
# -*- mode: python; coding: koi8-r; -*-
# (c) Con Radchenko mailto:bofh@pochta.ru
#


import fb2wrap


class FictionBook2Error(Exception):
    pass


class FictionBook2:

    _fb = None

    def __init__(self, filename):

        self._fb = fb2wrap.parseFile(filename)
        if self._fb == 'NULL':
            self._fb = None
            raise FictionBook2Error, 'Cannot parse file: ' + filename

    def __del__(self):
        if (self._fb):
            try:
                fb2wrap.freeFB2(self._fb)
            except:
                pass

    def __getattr__(self, name):
        if not self._fb:
            raise FictionBook2Error, 'fb2wrap is undefine'

        if name == 'text':
            return fb2wrap.FB2Content_text_get(self._fb)

        if name == 'description':
            return fb2wrap.FB2Content_description_get(self._fb)

        if name == 'author':
            return fb2wrap.FB2Content_author_get(self._fb)

        if name == 'name':
            return fb2wrap.FB2Content_name_get(self._fb)

        if name == 'bookmarks':
            bookmarks = []
            i = 0
            while 1:
                offset = fb2wrap.FB2Content_get_bookmark_offset(self._fb, i)
                if offset < 0: break
                length = fb2wrap.FB2Content_get_bookmark_length(self._fb, i)
                bookmarks.append((offset, length))
                i += 1

            return bookmarks

        if name == 'categories':
            categories = []
            i = 0
            while 1:
                cat = fb2wrap.FB2Content_get_category(self._fb, i)
                if not cat: break
                categories.append(cat)
                i += 1

            return categories

        raise AttributeError, name


if __name__ == '__main__':
    import sys
    if len(sys.argv) != 2:
        sys.exit('usage: %s file' % sys.argv[0])

    fb = FictionBook2(sys.argv[1])

    print fb
    print fb.categories
    print fb.author
    print fb.name
    text = fb.text
    for i in fb.bookmarks:
        print i, text[i[0]:i[0]+i[1]]
    print fb.description


