#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-

import gtk
from gtk import TRUE, FALSE

import string, gzip, os
import formatter, htmllib
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from utils import string_encode, error_dialog
from config import default_description_length, default_encoding, preview_length
from bookmarks import Book
from detectcharset import detectcharset
import filters

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class basic_parser:
    def new_book(self, filename):
        """Create new book object"""
        pass
    def load_file(self, buffer, book):
        """Load file in buffer"""
        pass
    def preview(self, buffer, book, offset):
        """Create preview: load part of file in buffer"""
        pass

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class plain_text:

##----------------------------------------------------------------------

    def new_book(self, filename, external_filter=None, encoding=None):
        """Create new book object"""

        book = Book()
        book.filename = filename

        if external_filter:
            book.externalfilter = external_filter
            if not encoding:
                fileobj = StringIO(self._read_from_pipe(filename,
                                                        external_filter))
                encoding = detectcharset(fileobj)
        else:
            fileobj = open(filename)
            encoding = detectcharset(fileobj)

        if encoding:
            book.encoding = encoding
        else:
            book.encoding = default_encoding

        book.file_type = 'plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------

    def load_file(self, buffer, book, bookmarks):
        """Load file in buffer"""

        data = self._read_file(filename=book.filename,
                               int_filters=book.internalfilters,
                               ext_filter=book.externalfilter,
                               charset=book.encoding, bookmarks=bookmarks)

        buffer.set_text(data)

##----------------------------------------------------------------------

    def preview(self, buffer, book, offset):
        """Create preview: load part of file in buffer"""

        data = self._read_file(filename=book.filename,
                               offset=offset, length=preview_length,
                               charset=book.encoding,
                               int_filters=book.internalfilters,
                               ext_filter=book.externalfilter)

        buffer.set_text(data)

##----------------------------------------------------------------------

    def _read_from_pipe(self, filename, command):

        if command.find('%s') > 0:
            command = command % filename
        else:
            command = command + ' ' + filename
        stdin, stdout, stderr = os.popen3(command)
        data = stdout.read()
        err = stderr.read()
        if err:
            error_dialog(None, '', err)

        return data

##----------------------------------------------------------------------

    def _read_file(self, filename, charset, offset=None, length=None,
                   int_filters=None, ext_filter=None, bookmarks=None):

        if ext_filter:
            data = self._read_from_pipe(filename, ext_filter)

            #pipe = os.popen(command)
            #data = pipe.read()
            #pipe.close()

        else:

            fd = open(filename)
            data = fd.read()
            fd.close()

        data = self._get_slice(data, charset, offset, length,
                               int_filters, bookmarks)

        return data

##----------------------------------------------------------------------
    # FIXME: rename
    def _get_slice(self, data, charset, offset=None, length=None,
                   int_filters=None, bookmarks=None):

        # delete invalid (\0) chars
        tbl = string.maketrans('', '')
        data = string.translate(data, tbl, '\0')

        data = unicode(data, charset, 'replace')
        if offset or length:
            if offset:
                data = data[offset:]
            if length:
                data = data[:length]
        data = data.encode('utf-8')

        if int_filters:
            for fltr in int_filters:
                func = getattr(filters, fltr)
                data = func(data, bookmarks=bookmarks)

        return data

##----------------------------------------------------------------------

    def book_description(self, book):
        """Create description of book"""

        data = self._read_file(filename=book.filename, charset=book.encoding,
                               length=default_description_length,
                               ext_filter=book.externalfilter)

        data += ' ...'

        return data

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class gzipped_plain_text(plain_text):

    def _read_file(self, filename, charset, offset=None, length=None,
                   int_filters=None, ext_filter=None, bookmarks=None):

        fd = gzip.open(filename)
        data = fd.read()
        fd.close()

        data = self._get_slice(data, charset, offset, length,
                               int_filters, bookmarks)

        return data

##----------------------------------------------------------------------

    def new_book(self, filename, external_filter=None, encoding=None):
        """Create new book object"""

        book = Book()
        book.filename = filename

        fileobj = gzip.open(filename)
        encoding = detectcharset(fileobj)
        if encoding:
            book.encoding = encoding
        else:
            book.encoding = default_encoding

        book.file_type = 'gzipped_plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class MyDumbWriter(formatter.NullWriter):

    def __init__(self, file=None, maxcol=72):
        self.data = []
        self.bookmarks = []
        formatter.NullWriter.__init__(self)

    def send_paragraph(self, blankline):
        self.data.append('\n'*blankline)

    def send_line_break(self):
        self.data.append('\n')

    def send_hor_rule(self, *args, **kw):
        self.data.append('\n')
        #self.data.append('\n')

    def send_literal_data(self, data):
        self.data.append(data)

    def send_flowing_data(self, data):
        if not data: return
        self.data.append(data)

    def new_font(self, font):
        if font and font[0]:
            self.bookmarks.append(reduce(lambda x, y: x+len(y), self.data, 0))

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class html_parser(plain_text):

    def new_book(self, filename, external_filter=None, encoding=None):
        """Create new book object"""

        book = Book()
        book.filename = filename

        if external_filter:
            book.externalfilter = external_filter
            fileobj = StringIO(self._read_from_pipe(filename, external_filter))
            if not encoding:
                encoding = detectcharset(fileobj)
        else:
            fileobj = open(filename)
            encoding = detectcharset(fileobj)

        if encoding:
            book.encoding = encoding
        else:
            encoding = book.encoding = default_encoding

        book.file_type = 'html_parser'
        book.description = self.book_description(book)

        return self._html_new_book(fileobj, book, encoding)

##----------------------------------------------------------------------

    def _html_new_book(self, fileobj, book, encoding):

        fileobj.seek(0)
        ex = fileobj.read()
        w = MyDumbWriter()
        h = htmllib.HTMLParser(formatter.AbstractFormatter(w))
        h.feed(ex)
        data = ''.join(w.data)
        if h.title:
            book.name = unicode(h.title, encoding, 'replace').encode('utf-8')

        text_len = len(data)
        data = unicode(data, encoding, 'replace')
        for bm in w.bookmarks:
            desc = data[bm:bm+default_description_length]
            desc = desc.encode('utf-8')
            book.add_bookmark(float(bm)/text_len, bm, desc)

        return book

##----------------------------------------------------------------------

    def _read_file(self, filename, charset, offset=None, length=None,
                   int_filters=None, ext_filter=None, bookmarks=None):

        if ext_filter:
            data = self._read_from_pipe(filename, ext_filter)
        else:
            data = open(filename).read()

        return self._parse_html(data, charset, length, offset,
                                int_filters, ext_filter, bookmarks)

#----------------------------------------------------------------------

    def _parse_html(self, ex, charset, length, offset,
                    int_filters, ext_filter, bookmarks):

        w = MyDumbWriter()
        h = htmllib.HTMLParser(formatter.AbstractFormatter(w))
        h.feed(ex)
        data = string.join(w.data, '')
        if len(h.anchorlist) > 0:
            data += '\n\nLinks:\n'
            n = 1
            for i in h.anchorlist:
                data += '  [%d] %s\n' % (n, i)
                n += 1

        data = self._get_slice(data, charset, offset, length,
                               int_filters, bookmarks)

        return data

##----------------------------------------------------------------------

class gzipped_html(html_parser):

    def new_book(self, filename, external_filter=None, encoding=None):
        """Create new book object"""

        book = Book()
        book.filename = filename

        fileobj = gzip.open(filename)
        encoding = detectcharset(fileobj)

        if encoding:
            book.encoding = encoding
        else:
            encoding = book.encoding = default_encoding

        book.file_type = 'gzipped_html'
        book.description = self.book_description(book)

        return self._html_new_book(fileobj, book, encoding)

##----------------------------------------------------------------------

    def _read_file(self, filename, charset, offset=None, length=None,
                   int_filters=None, ext_filter=None, bookmarks=None):

        fd = gzip.open(filename)
        data = fd.read()
        fd.close()

        return self._parse_html(data, charset, length, offset,
                                int_filters, ext_filter, bookmarks)

##----------------------------------------------------------------------
##----------------------------------------------------------------------
##----------------------------------------------------------------------

class LibRuWriter(formatter.NullWriter):

    def __init__(self, file=None, maxcol=72):
        self.data = []
        self.bookmarks = []
        formatter.NullWriter.__init__(self)

    def send_paragraph(self, blankline):
        self.data.append('\n'*blankline)

    def send_line_break(self):
        pass
        #self.data.append('\n')

    def send_hor_rule(self, *args, **kw):
        self.data.append('\n')
        #self.data.append('\n')

    def send_literal_data(self, data):
        self.data.append(data)

    def send_flowing_data(self, data):
        if not data: return
        if data[0] == ' ':
            self.data.append('\t')
        for i in data.strip().split():
            self.data.append(i+' ')

    def new_font(self, font):
        if font and font[0]:
            #self.bookmarks.append(len(' '.join(self.data)))
            self.bookmarks.append(reduce(lambda x, y: x+len(y), self.data, 0))

#----------------------------------------------------------------------

class lib_ru_parser(html_parser):

    def _parse_html(self, ex, charset, length, offset,
                    int_filters, ext_filter, bookmarks):

        w = LibRuWriter()
        h = htmllib.HTMLParser(formatter.AbstractFormatter(w))
        h.feed(ex)
        data = ''.join(w.data)
        if len(h.anchorlist) > 0:
            data += '\n\nLinks:\n'
            n = 1
            for i in h.anchorlist:
                data += '  [%d] %s\n' % (n, i)
                n += 1

        data = self._get_slice(data, charset, offset, length,
                               int_filters, bookmarks)

        return data


##----------------------------------------------------------------------
##----------------------------------------------------------------------

class fb2_parser(plain_text):

    def new_book(self, filename, external_filter=None, encoding=None):
        """Create new book object"""

        from fictionbook2 import FictionBook2

        book = Book()
        book.filename = filename
        book.encoding = 'utf-8'
        book.file_type = 'fb2_parser'

        fb = FictionBook2(filename)

        book.author = fb.author
        book.name = fb.name
        book.description = fb.description
        text = fb.text
        text_len = len(text)
        for b in fb.bookmarks:
            offset = len(unicode(text[:b[0]], 'utf-8', 'replace')) # this is ugly
            book.add_bookmark(float(b[0])/text_len, offset, text[b[0]:b[0]+b[1]])

        return book

##----------------------------------------------------------------------

    def _read_file(self, filename, charset, length=None, offset=None,
                   int_filters=None, ext_filter=None, bookmarks=None):

        from fictionbook2 import FictionBook2

        fb = FictionBook2(filename)

        data = self._get_slice(fb.text, 'utf-8', offset, length,
                               int_filters, bookmarks)

        return data

##----------------------------------------------------------------------
