#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os, sys, exceptions, codecs, gzip, locale
import gtk
from gtk import TRUE, FALSE

##----------------------------------------------------------------------
##----------------------------------------------------------------------

# FIXME: remove this class
class OpenFileError(exceptions.Exception):
    """OpenFileError exception"""
    def __init__(self, args=None):
        self.args = args

##----------------------------------------------------------------------

def get_glade_file():

    gf = 'pybookreader/pybookreader.glade'
    for d in sys.path:
        f = os.path.join(d, gf)
        if os.path.exists(f):
            return f
    return None

##----------------------------------------------------------------------

def string_encode(data, from_enc, to_enc='utf-8'):

    if from_enc == to_enc: return data

    inenc,  indec,  sr, sw = codecs.lookup(from_enc)
    outenc, outdec, sr, sw = codecs.lookup(to_enc)
    return outenc(indec(data, 'replace')[0], 'replace')[0]


##----------------------------------------------------------------------


def fix_description(data, len_desc):

    desc = ''
    #data = data.decode('utf-8', 'replace')
    for i in data:
        if string.find(string.whitespace, i) >= 0 or i == "'":
            if len(desc) != 0 and desc[len(desc)-1] != ' ':
                desc = desc + ' '
        else:
            desc = desc + i

        if len(desc) > len_desc and desc[len(desc)-1] == ' ': break

    desc = desc + '...' # ???

    #return desc.encode('utf-8', 'replace')
    return desc

##----------------------------------------------------------------------

def fix_filename(filename):
    enc = locale.getlocale()[1]
    fn = unicode(filename, enc).encode('utf-8')
    return fn

##----------------------------------------------------------------------

def entry_dialog(parent, title, label_text, text=''):

    dialog = gtk.Dialog(title, parent,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_OK, gtk.RESPONSE_OK,
                         gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
    hbox = gtk.HBox()
    dialog.vbox.pack_start(hbox, TRUE, TRUE, 0)
    label = gtk.Label(label_text)
    hbox.pack_start(label, expand=FALSE)
    entry = gtk.Entry()
    entry.set_text(text)
    entry.connect('activate', lambda w:
                  dialog.emit ('response', gtk.RESPONSE_OK))
    hbox.pack_start(entry)
    dialog.show_all()
    response = dialog.run()
    str = None
    if response == gtk.RESPONSE_OK:
        str = entry.get_text()
    dialog.destroy()
    return str

##----------------------------------------------------------------------

def text_dialog(parent, title, text=''):

    dialog = gtk.Dialog(title, parent,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_OK, gtk.RESPONSE_OK,
                         gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
    dialog.set_default_size(500, 300)  # FIXME: geometry

    scrolled_window = gtk.ScrolledWindow()
    scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    scrolled_window.set_border_width(4)
    scrolled_window.set_shadow_type(gtk.SHADOW_IN)
    dialog.vbox.pack_start(scrolled_window)

    text_view = gtk.TextView()
    text_buffer = gtk.TextBuffer()
    text_view.set_buffer(text_buffer)
    text_buffer.set_text(text)
    scrolled_window.add(text_view)

    dialog.show_all()
    response = dialog.run()
    str = None
    if response == gtk.RESPONSE_OK:
        start, end = text_buffer.get_bounds()
        str = text_buffer.get_text(start, end, TRUE)
    dialog.destroy()
    return str

##----------------------------------------------------------------------

def info_dialog(parent, msg):

    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL \
                               | gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                               msg)

    dialog.run()
    dialog.destroy()

##----------------------------------------------------------------------

def error_dialog(parent, msg, encoded_msg=None):

    if encoded_msg:
        enc = locale.getlocale()[1]
        msg += unicode(encoded_msg, enc, 'replace').encode('utf-8')

    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL \
                               | gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                               msg)

    dialog.run()
    dialog.destroy()

##----------------------------------------------------------------------

class OpenFileDialog:

    file_select_path = os.path.expanduser('~/')

    def __init__(self, win, select_multiple=None, save_as=None):

        dialog = gtk.FileSelection()
        dialog.set_transient_for(win)
        dialog.set_filename(self.file_select_path)
        dialog.show()
        self.filename = None
        self.files_list = None

        if select_multiple:
            dialog.set_select_multiple(TRUE)
            button = gtk.Button('Select _all')
            button.connect('clicked', lambda w:
                           dialog.file_list.get_selection().select_all())
            dialog.button_area.pack_end(button)
            button.show()

        while 1:

            response = dialog.run()
            if response == gtk.RESPONSE_OK:

                filename = dialog.get_filename()

                if os.path.isdir(filename):
                    model, iter = dialog.dir_list.get_selection().get_selected()
                    if iter:
                        dirname = os.path.normpath(filename
                                                   + model.get_value(iter, 0)) \
                                                   + os.sep
                    else:
                        dirname = os.path.normpath(filename) + os.sep
                    OpenFileDialog.file_select_path = dirname
                    dialog.set_filename(dirname)
                    continue

                if select_multiple:

                    OpenFileDialog.file_select_path \
                        = os.path.dirname(filename) + os.sep

                    sel = dialog.file_list.get_selection()
                    files_list = []
                    sel.selected_foreach(lambda model, path, iter:
                        files_list.append(model.get_value(iter, 0)))
                    self.filename = filename
                    self.files_list = files_list
                    dialog.destroy()
                    return

                elif save_as:
                    if os.path.exists(filename):

                        md = gtk.MessageDialog(dialog, gtk.DIALOG_MODAL \
                                               | gtk.DIALOG_DESTROY_WITH_PARENT,
                                               gtk.MESSAGE_WARNING,
                                               gtk.BUTTONS_OK_CANCEL,
                                               'File %s exists. Overwrite?' \
                                               % filename)

                        response = md.run()
                        if response == gtk.RESPONSE_OK:
                            md.destroy()
                            self.filename = filename
                        else:
                            md.destroy()
                            continue
                    else:
                        self.filename = filename

                    dialog.destroy()
                    return

                else:

                    try:
                        file = open(filename)
                    except IOError, err:
                        error_dialog(dialog, "Can't open file %s: %s" \
                                     % (filename, err[1]))
                        continue
                    else:
                        file.close()
                        self.filename = filename
                        OpenFileDialog.file_select_path \
                            = os.path.dirname(filename) + os.sep
                        dialog.destroy()
                        return

            else:
                dialog.destroy()
                return

##----------------------------------------------------------------------
